/*	Application		A program which continually requests more memory until it is refused
	File			Memory_Hog.cpp 
	Purpose			A Memory-intensive program which is used to demonstrate attributes of memory management behaviour
	Author			Richard John Anthony
	Date			December 2013
	
	Function		1. Runs in a loop requesting memory, finds allocation limit by doubling request size until a request is refused
					2. Requests and holds the maximum allocation memory amount discovered.
					3. Displays the amount of memory held
*/


#include "stdafx.h"
#include <malloc.h>
#include <iostream>

using namespace std;

#define AllocationIncrement 1000000	// 1 million bytes


int _tmain(int argc, _TCHAR* argv[])
{
	char *pMemoryAllocated;
	int iAllocationCount = 1;
	char cIn;
	cout << "*******************************************************************************" << endl;
	cout << "*   WARNING   -  could crash your computer or loose data  -  read carefully   *" << endl;
	cout << "*******************************************************************************" << endl;
	cout << "* This program has been designed for the exploration of memory allocation     *" << endl;
	cout << "* behaviour. Running this program COULD cause problems with your computer -   *" << endl;
	cout << "* do not run this program unless you are happy to risk a crash.               *" << endl;
	cout << "* If you do decide to continue to run this program, make sure you have saved  *" << endl;
	cout << "* all work, closed all files and shutdown any applications which may corrupt  *" << endl;
	cout << "* data. This program should not be run on any computer containing valuable    *" << endl;
	cout << "* data resources or running important services; if in doubt DO NOT RUN IT.    *" << endl;
	cout << "* The results of running multiple copies of this program, for the purposes of *" << endl;
	cout << "* exploring the operating system's behaviour under extreme memory demand      *" << endl;
	cout << "* conditions are reported in the accompanying book, in the Resource-View      *" << endl;
	cout << "* chapter, so you can read the results and accompanying explanation without   *" << endl;
	cout << "* having to actually run this program.                                        *" << endl;
	cout << "*******************************************************************************" << endl;
	cout << "*                        To continue with this program                        *" << endl;
	cout << "*      (which involves dynamic allocation of very large blocks of memory)     *" << endl;
	cout << "*             press 'M' and then the 'enter' key followed by '@'              *" << endl;
	cout << "*                                                                             *" << endl;
	cout << "*         To exit now without performing any dynamic memory allocation        *" << endl;
	cout << "*                press any other key and then the 'enter' key                 *" << endl;
	cout << "*******************************************************************************" << endl;
	
	cin >> cIn;
	if('M' != cIn)
	{
		cout << "Exiting safely" << endl;
		exit(0);
	}
	cin >> cIn;
	if('@' != cIn)
	{
		cout << "Exiting safely" << endl;
		exit(0);
	}


	do
	{
		cout << "Requesting " << iAllocationCount << " * " << AllocationIncrement << " bytes";

		//pMemoryAllocated = (char *)malloc(AllocationIncrement * iAllocationCount * sizeof (*pMemoryAllocated));
		pMemoryAllocated = reinterpret_cast<char *>(malloc(AllocationIncrement * iAllocationCount * sizeof (*pMemoryAllocated))); /* with a cast, for C++ */
		if(nullptr != pMemoryAllocated)
		{
			free(pMemoryAllocated);
			cout << " ALLOCATED (now freeing)" << endl;
			iAllocationCount *= 2;
		}
		else
		{	
			cout << " REFUSED" << endl;
			iAllocationCount /= 2;
			//pMemoryAllocated = (char *)malloc(AllocationIncrement * iAllocationCount * sizeof (*pMemoryAllocated));
			pMemoryAllocated = reinterpret_cast<char *>(malloc(AllocationIncrement * iAllocationCount * sizeof (*pMemoryAllocated))); /* with a cast, for C++ */
			if(nullptr != pMemoryAllocated)
			{
				cout << iAllocationCount << " * " << AllocationIncrement << " bytes re-allocated and held" << endl;
			}
			else
			{	
				cout << "Re-allocation attempt failed - No memory held" << endl;
			}
			break;
		}
	} while (nullptr != pMemoryAllocated);

	while(1)
	{
		// Wait here to allow exploration, e.g. with Task Manager
	}

	return 0;
}

